# cat nodes

原文链接 : [https://www.elastic.co/guide/en/elasticsearch/reference/5.0/cat-nodes.html](https://www.elastic.co/guide/en/elasticsearch/reference/5.0/cat-nodes.html)

译文链接 : [http://www.apache.wiki/display/Elasticsearch/cat+nodes](http://www.apache.wiki/display/Elasticsearch/cat+nodes)

贡献者 : [那伊抹微笑](/display/~wangyangting)

**nodes** 命令展示了集群拓扑结构。

```
% GET /_cat/nodes
192.168.56.30 9 78 22 1.80 2.05 2.51 mdi * bGG90GE
192.168.56.10 6 75 14 1.24 2.45 1.37 md  - I8hydUG
192.168.56.20 5 71 12 1.07 1.05 1.11 di  - H5dfFeA
```

前几列告诉您可用的节点并且描述了一下 **heap**（堆），**memory**（内存），**cpu**，和 **load**（平均负载）。

```
ip            heap.percent ram.percent cpu load_1m load_5m load_15m
192.168.56.30            9          78  22    1.80    2.05     2.51
192.168.56.10            6          75  14    1.24    2.45     1.37
192.168.56.20            5          71  12    1.07    1.05     1.11
```

最后几列提供了在查看整个集群（特别是大集群）时的一些有用的辅助信息。我有多少个 **master-eligible**（**master** 资格的）节点。

## Columns（列）

下面是一个已存在的 **header**（标题）的详细列表，它可以传递 **nodeattrs?h=** 以在有序的列中获取更详细的信息。如果没有指定 **header**（标题），那么这些标记默认情况下就会有。如果指定了 **header**（标题），将不使用默认值。

为了更简洁，可以使用别名替换完整的标题名称。列的显示是有序的，它们在下面被列出了，除非指定了其它的顺序（例如，**h=attr**，**value **与 **h=value**，**attr** 相反）。

在指定标题时，默认的输出中标题不会被替换。为了使标题可以出现在输出中，使用 **verbose**（详细的）模式（**v**）。标题名将匹配提供的值（例如，**pid** 与 **p** 相对）。例如 : 

```
% curl 192.168.56.10:9200/_cat/nodes?v&h=id,ip,port,v,m
id   ip            port v           m
pLSN 192.168.56.30 9300 {version} -
k0zy 192.168.56.10 9300 {version} -
6Tyi 192.168.56.20 9300 {version} *
% curl 192.168.56.10:9200/_cat/nodes?h=id,ip,port,v,m
pLSN 192.168.56.30 9300 {version} -
k0zy 192.168.56.10 9300 {version} -
6Tyi 192.168.56.20 9300 {version} *
```

| Header（标题） | Alias（别名） | Appear by Default（默认显示） | Description（描述） | Example（示例） |
| --- | --- | --- | --- | --- |
| `id` | `nodeId` | No | Unique node ID | k0zy |
| `pid` | `p` | No | Process ID | 13061 |
| `ip` | `i` | Yes | IP address | 127.0.1.1 |
| `port` | `po` | No | Bound transport port | 9300 |
| `http_address` | `http` | No | bound http address | 127.0.0.1:9200 |
| `version` | `v` | No | Elasticsearch version | 5.0.1 |
| `build` | `b` | No | Elasticsearch Build hash | 5c03844 |
| `jdk` | `j` | No | Running Java version | 1.8.0 |
| `disk.avail` | `d`, `disk`, `diskAvail` | No | Available disk space | 1.8gb |
| `heap.current` | `hc`, `heapCurrent` | No | Used heap | 311.2mb |
| `heap.percent` | `hp`, `heapPercent` | Yes | Used heap percentage | 7 |
| `heap.max` | `hm`, `heapMax` | No | Maximum configured heap | 1015.6mb |
| `ram.current` | `rc`, `ramCurrent` | No | Used total memory | 513.4mb |
| `ram.percent` | `rp`, `ramPercent` | Yes | Used total memory percentage | 47 |
| `ram.max` | `rm`, `ramMax` | No | Total memory | 2.9gb |
| `file_desc.current` | `fdc`,`fileDescriptorCurrent` | No | Used file descriptors | 123 |
| `file_desc.percent` | `fdp`,`fileDescriptorPercent` | Yes | Used file descriptors percentage | 1 |
| `file_desc.max` | `fdm`, `fileDescriptorMax` | No | Maximum number of file descriptors | 1024 |
| `cpu` |   | No | Recent system CPU usage as percent | 12 |
| `load_1m` | `l` | No | Most recent load average | 0.22 |
| `load_5m` | `l` | No | Load average for the last five minutes | 0.78 |
| `load_15m` | `l` | No | Load average for the last fifteen minutes | 1.24 |
| `uptime` | `u` | No | Node uptime | 17.3m |
| `node.role` | `r`, `role`, `nodeRole` | Yes | Master eligible node (m); Data node (d); Ingest node (i); Coordinating node only (-) | mdi |
| `master` | `m` | Yes | Elected master (*); Not elected master (-) | * |
| `name` | `n` | Yes | Node name | I8hydUG |
| `completion.size` | `cs`, `completionSize` | No | Size of completion | 0b |
| `fielddata.memory_size` | `fm`, `fielddataMemory` | No | Used fielddata cache memory | 0b |
| `fielddata.evictions` | `fe`, `fielddataEvictions` | No | Fielddata cache evictions | 0 |
| `query_cache.memory_size` | `qcm`, `queryCacheMemory` | No | Used query cache memory | 0b |
| `query_cache.evictions` | `qce`, `queryCacheEvictions` | No | Query cache evictions | 0 |
| `request_cache.memory_size` | `rcm`, `requestCacheMemory` | No | Used request cache memory | 0b |
| `request_cache.evictions` | `rce`,`requestCacheEvictions` | No | Request cache evictions | 0 |
| `request_cache.hit_count` | `rchc`,`requestCacheHitCount` | No | Request cache hit count | 0 |
| `request_cache.miss_count` | `rcmc`,`requestCacheMissCount` | No | Request cache miss count | 0 |
| `flush.total` | `ft`, `flushTotal` | No | Number of flushes | 1 |
| `flush.total_time` | `ftt`, `flushTotalTime` | No | Time spent in flush | 1 |
| `get.current` | `gc`, `getCurrent` | No | Number of current get operations | 0 |
| `get.time` | `gti`, `getTime` | No | Time spent in get | 14ms |
| `get.total` | `gto`, `getTotal` | No | Number of get operations | 2 |
| `get.exists_time` | `geti`, `getExistsTime` | No | Time spent in successful gets | 14ms |
| `get.exists_total` | `geto`, `getExistsTotal` | No | Number of successful get operations | 2 |
| `get.missing_time` | `gmti`, `getMissingTime` | No | Time spent in failed gets | 0s |
| `get.missing_total` | `gmto`, `getMissingTotal` | No | Number of failed get operations | 1 |
| `indexing.delete_current` | `idc`,`indexingDeleteCurrent` | No | Number of current deletion operations | 0 |
| `indexing.delete_time` | `idti`, `indexingDeleteTime` | No | Time spent in deletions | 2ms |
| `indexing.delete_total` | `idto`, `indexingDeleteTotal` | No | Number of deletion operations | 2 |
| `indexing.index_current` | `iic`, `indexingIndexCurrent` | No | Number of current indexing operations | 0 |
| `indexing.index_time` | `iiti`, `indexingIndexTime` | No | Time spent in indexing | 134ms |
| `indexing.index_total` | `iito`, `indexingIndexTotal` | No | Number of indexing operations | 1 |
| `indexing.index_failed` | `iif`, `indexingIndexFailed` | No | Number of failed indexing operations | 0 |
| `merges.current` | `mc`, `mergesCurrent` | No | Number of current merge operations | 0 |
| `merges.current_docs` | `mcd`, `mergesCurrentDocs` | No | Number of current merging documents | 0 |
| `merges.current_size` | `mcs`, `mergesCurrentSize` | No | Size of current merges | 0b |
| `merges.total` | `mt`, `mergesTotal` | No | Number of completed merge operations | 0 |
| `merges.total_docs` | `mtd`, `mergesTotalDocs` | No | Number of merged documents | 0 |
| `merges.total_size` | `mts`, `mergesTotalSize` | No | Size of current merges | 0b |
| `merges.total_time` | `mtt`, `mergesTotalTime` | No | Time spent merging documents | 0s |
| `refresh.total` | `rto`, `refreshTotal` | No | Number of refreshes | 16 |
| `refresh.time` | `rti`, `refreshTime` | No | Time spent in refreshes | 91ms |
| `script.compilations` | `scrcc`, `scriptCompilations` | No | Total script compilations | 17 |
| `script.cache_evictions` | `scrce`,`scriptCacheEvictions` | No | Total compiled scripts evicted from cache | 6 |
| `search.fetch_current` | `sfc`, `searchFetchCurrent` | No | Current fetch phase operations | 0 |
| `search.fetch_time` | `sfti`, `searchFetchTime` | No | Time spent in fetch phase | 37ms |
| `search.fetch_total` | `sfto`, `searchFetchTotal` | No | Number of fetch operations | 7 |
| `search.open_contexts` | `so`, `searchOpenContexts` | No | Open search contexts | 0 |
| `search.query_current` | `sqc`, `searchFetchCurrent` | No | Current query phase operations | 0 |
| `search.query_time` | `sqti`, `searchFetchTime` | No | Time spent in query phase | 43ms |
| `search.query_total` | `sqto`, `searchFetchTotal` | No | Number of query operations | 9 |
| `search.scroll_current` | `scc`, `searchScrollCurrent` | No | Open scroll contexts | 2 |
| `search.scroll_time` | `scti`, `searchScrollTime` | No | Time scroll contexts held open | 2m |
| `search.scroll_total` | `scto`, `searchScrollTotal` | No | Completed scroll contexts | 1 |
| `segments.count` | `sc`, `segmentsCount` | No | Number of segments | 4 |
| `segments.memory` | `sm`, `segmentsMemory` | No | Memory used by segments | 1.4kb |
| `segments.index_writer_memory` | `siwm`,`segmentsIndexWriterMemory` | No | Memory used by index writer | 18mb |
| `segments.version_map_memory` | `svmm`,`segmentsVersionMapMemory` | No | Memory used by version map | 1.0kb |
| `segments.fixed_bitset_memory` | `sfbm`, `fixedBitsetMemory` | No | Memory used by fixed bit sets for nested object field types and type filters for types referred in _parent fields | 1.0kb |
| `suggest.current` | `suc`, `suggestCurrent` | No | Number of current suggest operations | 0 |
| `suggest.time` | `suti`, `suggestTime` | No | Time spent in suggest | 0 |
| suggest.total | `suto`, `suggestTotal` | No | Number of suggest operations | 0 |